/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */

package relayboard;

import java.io.IOException;
import java.util.logging.Level;
import java.util.logging.Logger;
import jd2xx.JD2XX;
import static jd2xx.JD2XX.*;

/**
 *
 * @author Matt
 */

public class RelayBoard {
    public enum CycleMode {
        BINARYCOUNTER, SHIFTONE, KNIGHTRIDER, SHIFTFOUR
    };

    private static final int RELAYNUMBER_MIN=1;
    private static final int RELAYNUMBER_MAX=8;    
    public static final int CYCLEDELAY_MS_MIN=100;
    
    private static JD2XX board = new JD2XX();
    private int currentValue=0;
    private int relayCount=4;
    public String serialNumber="";
    /**
     * @param args the command line arguments
     */
    
    public RelayBoard(String serialNumber, int numRelays) {
        this.serialNumber = serialNumber;
        if (numRelays>=RELAYNUMBER_MIN && numRelays<=RELAYNUMBER_MAX) {
            this.relayCount = numRelays;
        } else {
            System.out.println("invalid number of relays.  using default: "+this.relayCount);
        }
    }
    
    public int open() {
        int status=1;
        if (!this.serialNumber.equals("")) {
            try {
                board.openBySerialNumber(this.serialNumber);
                this.setBitMode();
            } catch (IOException ex) {
                Logger.getLogger(RelayBoard.class.getName()).log(Level.SEVERE, null, ex);
                status=0;
            }
        }
        return status;
    }
    
    private int setBitMode() {
        int status=1;
        try {
            board.setBitMode(255, BITMODE_ASYNC_BITBANG);
        } catch (IOException ex) {
            Logger.getLogger(RelayBoard.class.getName()).log(Level.SEVERE, null, ex);
            status=0;
        }
        return status;
    }
    
    public int close() {
        int status=1;
        try {
            board.close();
        } catch (IOException ex) {
            Logger.getLogger(RelayBoard.class.getName()).log(Level.SEVERE, null, ex);
            status=0;
        }
        return status;
    }
    
    private int sendRawValue(int val) {
        int status=1;
        int maxVal = (int) (Math.pow(2,RELAYNUMBER_MAX)-1);
        if (val<0 || val>maxVal) {
            System.out.println("invalid value: "+val+". must be a number 0 through "+maxVal);
            status=-1;
        } else {
            try {
                //this.open();
                //this.setBitMode();
                board.write(val);
                //this.close();
                this.currentValue=val;
            } catch (IOException ex) {
                Logger.getLogger(RelayBoard.class.getName()).log(Level.SEVERE, null, ex);
                status=0;
            }
        }
        return status;
    }

    public int turnOnAll() {
        int status;
        status = sendRawValue((int) (Math.pow(2,this.relayCount)-1));
        return status;
    }
    
    public int turnOffAll() {
        int status;
        status = sendRawValue(0);
        return status;
    }

    public int turnOn(int relayNumber) {
        int status=1;
        int val=0;
        if (relayNumber<RELAYNUMBER_MIN || relayNumber>RELAYNUMBER_MAX) {
            System.out.println("invalid relay number: "+relayNumber+".  must be a number 1 through 8. ");
            status=0;
        } else {
            val = this.currentValue | (1<<(relayNumber-1));
            status = sendRawValue(val);
        }
        return status;
    }
    
    public int turnOff(int relayNumber) {
        int status=1;
        int val=0;
        if (relayNumber<RELAYNUMBER_MIN || relayNumber>RELAYNUMBER_MAX) {
            System.out.println("invalid relay number: "+relayNumber+".  must be a number 1 through 8. ");
            status=0;
        } else {
            val = this.currentValue & ~(1<<(relayNumber-1));
            status = sendRawValue(val);
        }
        return status;
    }

    public void cycle(int numberOfCycles, int delay_ms, CycleMode mode) {
        if (numberOfCycles<1) {
            System.out.println("invalid number of cycles: "+numberOfCycles);
            return;
        }
        if (delay_ms<CYCLEDELAY_MS_MIN) {
            System.out.println("invalid delay time: "+delay_ms+". must be >="+CYCLEDELAY_MS_MIN);
            return;
        }

        try {
            this.sendRawValue(0);
            for (int cycle=0;cycle<numberOfCycles;cycle++) {
                switch (mode) {
                    case BINARYCOUNTER:
                        for (int i=0;i<((int) Math.pow(2,this.relayCount));i++) {
                            this.sendRawValue(i);
                            Thread.sleep(delay_ms);
                        }
                        break;
                    case SHIFTONE:
                        for (int i=0;i<this.relayCount;i++) {
                            this.sendRawValue(1<<i);
                            Thread.sleep(delay_ms);
                        }
                        break;
                    case KNIGHTRIDER:
                        if (cycle==0) {
                            this.sendRawValue(1);
                            Thread.sleep(delay_ms);
                        }
                        for (int i=1;i<this.relayCount;i++) {
                            this.sendRawValue(1<<i);
                            Thread.sleep(delay_ms);
                        }
                        for (int i=this.relayCount-2;i>=0;i--) {
                            this.sendRawValue(1<<i);
                            Thread.sleep(delay_ms);
                        }
                        break;
                    case SHIFTFOUR:
                        for (int i=1;i<=this.relayCount;i++) {
                            this.turnOn(i);
                            Thread.sleep(delay_ms);
                        }
                        for (int i=1;i<=this.relayCount;i++) {
                            this.turnOff(i);
                            Thread.sleep(delay_ms);
                        }
                        break;
                }
            }
            this.sendRawValue(0);
        } catch (InterruptedException ex) {
            Logger.getLogger(RelayBoard.class.getName()).log(Level.SEVERE, null, ex);
        }
    }

/*    public static void main(String[] args) throws InterruptedException {
        try {
            devs = board.listDevicesBySerialNumber();
            if (devs.length>0) {
                board.openBySerialNumber(devs[0].toString());
                board.setBitMode(255, BITMODE_ASYNC_BITBANG);
            }
        } catch (IOException ex) {
            Logger.getLogger(RelayBoard.class.getName()).log(Level.SEVERE, null, ex);
        }
        
    }
*/
}
